// Copyright 2016, University of Colorado Boulder

// Loaded by classroom-activity.js
(function() {
  'use strict';

  /**
   * Evaluates if testing is allowed by comparing the current tab with the active (furthest progressed tab)
   * @returns {boolean} - if testing is allowed, true
   */
  function isTestingAllowed() {
    if ( window.classroom.currentTabId === 'experiment' ) {
      return window.classroom.experiment.inTest && !window.classroom.experiment.ideas[ window.classroom.experiment.currentIdeaIndex ].tested;
    }
    else {
      return window.classroom.activeTabId === window.classroom.currentTabId;
    }
  }

  /**
   * The main object that dictates the specific forces-and-motion-basics customizations in the classroom activity
   * @type {{pageTitle: string, queryString: string, predictText1: string, predictText2: string, hypothesisChoices: *[], exploreText: string, experimentText1: string, reflectText1: string, reflectText2: string, reflectText3: string, applyText1: string, applyText2: string, applyText3: string, customization: {predict: *[], explore: *[], experiment: *[], reflect: *[], apply: Array, applyCases: *[], disablePullers: *[], enablePullers: *[], setInactiveAndNotInteractive: *[], setActiveAndInteractive: *[], applyPredictions: *[], applyRestoreCase: *[]}, getApplyOnLoadCustomizations: Window.simConfiguration.getApplyOnLoadCustomizations, modelTandem: string, activePropertyTandem: string, initializeExperimentListeners: Window.simConfiguration.initializeSimInteractionListeners, getIdeaCustomizations: Window.simConfiguration.getIdeaCustomizations, experimentTableHeaders: *[], getExperimentCellSetters: Window.simConfiguration.getExperimentCellSetters, applyTableHeaders: *[], getApplyCellSetters: Window.simConfiguration.getApplyCellSetters, currentDataPoint: {leftForce: number, rightForce: number, netForce: number, duration: number}, started: boolean, state: string, activeListener: Window.simConfiguration.activeListener, startedListener: Window.simConfiguration.startedListener, stateListener: Window.simConfiguration.stateListener, runningListener: Window.simConfiguration.runningListener, getForceAndDuration: Window.simConfiguration.getForceAndDuration, recordDataPoint: Window.simConfiguration.recordDataPoint}}
   */
  window.simConfiguration = {

    pageTitle: 'PhET-iO Classroom Activity | Forces and Motion Basics',

    queryString: 'screens=1',

    // The text for the activity, filled in dynamically
    predictText1: 'Decide if the blue team wins, the red team wins, or the cart doesn\'t move.',
    predictText2: 'Explain why you think the cart will move in the direction you chose.',
    hypothesisChoices: [
      { value: 'red', text: 'Red team wins' },
      { value: 'blue', text: 'Blue team wins' },
      { value: 'tie', text: 'Cart doesn\'t move' }
    ],
    exploreText: 'Write at least three things you notice about how force affects the cart motion in the simulation.',
    experimentText1: 'Figure out what makes a team win. Can you predict who will win, and why, for any set up?',
    experimentText2: 'Use the Save Trial button to fill in the data table.',
    experimentText3: 'Organize your data to support your conclusions.',

    reflectText1: 'How do you know which team will win?',
    reflectText2: 'Write a set of rules to predict which team wins for any setup.',
    reflectText3: 'Collect more data in the table if needed.',

    applyText1: 'Use your rules to predict who wins.',
    applyText2: 'Explain your reasoning.',
    applyText3: 'When you have made prediction for all 4 cases, submit your answers using the button.',
    applyText4: 'Test your predictions using the simulation.',
    applyText5: 'Use the play button in the table to restore each case.',
    applyText6: 'Edit your rules if needed after testing your predictions.',

    //@formatter:off

    // These customizations hold the tandems to interact with the sim depending on the state desired
    customization: {
      predict: [
        // Hide checkbox panel, volume control, and reset all button
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.controlPanel', method: 'setVisible', args: [ false ] },
        // Hide Return button
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.returnButton', method: 'setVisible', args: [ false ] },
        // Hide left force arrow
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.leftArrow', method: 'setVisible', args: [ false ] },
        // Hide right force arrow
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.rightArrow', method: 'setVisible', args: [ false ] },
        // Disable the sim
        { phetioID: 'forcesAndMotionBasics.sim.activeProperty', method: 'setValue', args: [ false ] },
        // Set Blue puller knot property (puller position is linked)
        { phetioID: 'forcesAndMotionBasics.netForceScreen.model.mediumBluePuller.knotProperty', method: 'setValue', args: [ 'forcesAndMotionBasics.netForceScreen.model.blueKnot2' ] },
        // Set Red puller knot property (puller position is linked)
        { phetioID: 'forcesAndMotionBasics.netForceScreen.model.largeRedPuller.knotProperty', method: 'setValue', args: [ 'forcesAndMotionBasics.netForceScreen.model.redKnot1' ] },
        // Hide Go button
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.goPauseButton', method: 'setVisible', args: [ false ] }
      ],

      explore: [
        // Reset pullers to original location
        { phetioID: 'forcesAndMotionBasics.netForceScreen.model', method: 'reset', args: [] },
        // Show Checkbox panel, volume control, and reset all button
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.controlPanel', method: 'setVisible', args: [ true ] },
        // Go button
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.goPauseButton', method: 'setVisible', args: [ true ] },
        // Return button
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.returnButton', method: 'setVisible', args: [ true ] },
        // Show  left force arrow
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.leftArrow', method: 'setVisible', args: [ true ] },
        // Show right force arrow
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.rightArrow', method: 'setVisible', args: [ true ] },
        // Enable the sim
        { phetioID: 'forcesAndMotionBasics.sim.activeProperty', method: 'setValue', args: [ true ] },
        //  Values box checked
        { phetioID: 'forcesAndMotionBasics.netForceScreen.model.showValuesProperty', method: 'setValue', args: [ true ] }
      ],

      experiment: [
        // Reset pullers to original location
        { phetioID: 'forcesAndMotionBasics.netForceScreen.model', method: 'reset', args: [] },
        // Show Checkbox panel, volume control, and reset all button
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.controlPanel', method: 'setVisible', args: [ true ] },
        // Go button
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.goPauseButton', method: 'setVisible', args: [ true ] },
        // Return button
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.returnButton', method: 'setVisible', args: [ true ] },
        // Show  left force arrow
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.leftArrow', method: 'setVisible', args: [ true ] },
        // Show right force arrow
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.rightArrow', method: 'setVisible', args: [ true ] },
        // Disable the sim
        { phetioID: 'forcesAndMotionBasics.sim.activeProperty', method: 'setValue', args: [ false ] }
      ],

      reflect: [
        // Reset pullers to original location
        { phetioID: 'forcesAndMotionBasics.netForceScreen.model', method: 'reset', args: [] },
        // Show Checkbox panel, volume control, and reset all button
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.controlPanel', method: 'setVisible', args: [ true ] },
        // Go button
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.goPauseButton', method: 'setVisible', args: [ true ] },
        // Return button
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.returnButton', method: 'setVisible', args: [ true ] },
        // Show  left force arrow
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.leftArrow', method: 'setVisible', args: [ true ] },
        // Show right force arrow
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.rightArrow', method: 'setVisible', args: [ true ] },
        // Show force values
        { phetioID: 'forcesAndMotionBasics.netForceScreen.model.showValuesProperty', method: 'setValue', args: [ true ] }
      ],

      // Empty array needed to be called automatically by classroom-activity.js:switchTab()
      apply: [],

      // Set puller positions for the apply cases
      applyCases: [
        // [
        //   { phetioID: 'forcesAndMotionBasics.netForceScreen.model.mediumBluePuller.knotProperty', method: 'setValue', args: [ 'forcesAndMotionBasics.netForceScreen.model.blueKnot2' ] },
        //   { phetioID: 'forcesAndMotionBasics.netForceScreen.model.largeRedPuller.knotProperty', method: 'setValue', args: [ 'forcesAndMotionBasics.netForceScreen.model.redKnot1' ] },
        // ],
        [
          { phetioID: 'forcesAndMotionBasics.netForceScreen.model.largeBluePuller.knotProperty', method: 'setValue', args: [ 'forcesAndMotionBasics.netForceScreen.model.blueKnot0' ] },
          { phetioID: 'forcesAndMotionBasics.netForceScreen.model.largeRedPuller.knotProperty', method: 'setValue', args: [ 'forcesAndMotionBasics.netForceScreen.model.redKnot0' ] }
        ],
        // [
        //   { phetioID: 'forcesAndMotionBasics.netForceScreen.model.smallBluePuller1.knotProperty', method: 'setValue', args: [ 'forcesAndMotionBasics.netForceScreen.model.blueKnot1' ] },
        //   { phetioID: 'forcesAndMotionBasics.netForceScreen.model.smallBluePuller2.knotProperty', method: 'setValue', args: [ 'forcesAndMotionBasics.netForceScreen.model.blueKnot3' ] },
        //   { phetioID: 'forcesAndMotionBasics.netForceScreen.model.mediumRedPuller.knotProperty', method: 'setValue', args: [ 'forcesAndMotionBasics.netForceScreen.model.redKnot1' ] },
        // ],
        [
          { phetioID: 'forcesAndMotionBasics.netForceScreen.model.largeBluePuller.knotProperty', method: 'setValue', args: [ 'forcesAndMotionBasics.netForceScreen.model.blueKnot1' ] },
          { phetioID: 'forcesAndMotionBasics.netForceScreen.model.smallRedPuller1.knotProperty', method: 'setValue', args: [ 'forcesAndMotionBasics.netForceScreen.model.redKnot0' ] },
          { phetioID: 'forcesAndMotionBasics.netForceScreen.model.smallRedPuller2.knotProperty', method: 'setValue', args: [ 'forcesAndMotionBasics.netForceScreen.model.redKnot1' ] }
        ],
        [
          { phetioID: 'forcesAndMotionBasics.netForceScreen.model.smallBluePuller1.knotProperty', method: 'setValue', args: [ 'forcesAndMotionBasics.netForceScreen.model.blueKnot3' ] },
          { phetioID: 'forcesAndMotionBasics.netForceScreen.model.smallBluePuller2.knotProperty', method: 'setValue', args: [ 'forcesAndMotionBasics.netForceScreen.model.blueKnot2' ] },
          { phetioID: 'forcesAndMotionBasics.netForceScreen.model.largeBluePuller.knotProperty', method: 'setValue', args: [ 'forcesAndMotionBasics.netForceScreen.model.blueKnot0' ] },
          { phetioID: 'forcesAndMotionBasics.netForceScreen.model.mediumRedPuller.knotProperty', method: 'setValue', args: [ 'forcesAndMotionBasics.netForceScreen.model.redKnot1' ] },
          { phetioID: 'forcesAndMotionBasics.netForceScreen.model.largeRedPuller.knotProperty', method: 'setValue', args: [ 'forcesAndMotionBasics.netForceScreen.model.redKnot3' ] }
        ],
        [
          { phetioID: 'forcesAndMotionBasics.netForceScreen.model.largeBluePuller.knotProperty', method: 'setValue', args: [ 'forcesAndMotionBasics.netForceScreen.model.blueKnot1' ] },
          { phetioID: 'forcesAndMotionBasics.netForceScreen.model.smallRedPuller1.knotProperty', method: 'setValue', args: [ 'forcesAndMotionBasics.netForceScreen.model.redKnot0' ] },
          { phetioID: 'forcesAndMotionBasics.netForceScreen.model.smallRedPuller2.knotProperty', method: 'setValue', args: [ 'forcesAndMotionBasics.netForceScreen.model.redKnot1' ] },
          { phetioID: 'forcesAndMotionBasics.netForceScreen.model.mediumRedPuller.knotProperty', method: 'setValue', args: [ 'forcesAndMotionBasics.netForceScreen.model.redKnot2' ] }
        ]


      ],

      disablePullers: [
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.smallBluePuller1', method: 'setPickable', args: [ false ] },
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.smallBluePuller2', method: 'setPickable', args: [ false ] },
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.smallRedPuller1', method: 'setPickable', args: [ false ] },
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.smallRedPuller2', method: 'setPickable', args: [ false ] },
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.mediumBluePuller', method: 'setPickable', args: [ false ] },
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.mediumRedPuller', method: 'setPickable', args: [ false ] },
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.largeBluePuller', method: 'setPickable', args: [ false ] },
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.largeRedPuller', method: 'setPickable', args: [ false ] }
      ],

      enablePullers: [
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.smallBluePuller1', method: 'setPickable', args: [ true ] },
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.smallBluePuller2', method: 'setPickable', args: [ true ] },
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.smallRedPuller1', method: 'setPickable', args: [ true ] },
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.smallRedPuller2', method: 'setPickable', args: [ true ] },
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.mediumBluePuller', method: 'setPickable', args: [ true ] },
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.mediumRedPuller', method: 'setPickable', args: [ true ] },
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.largeBluePuller', method: 'setPickable', args: [ true ] },
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.largeRedPuller', method: 'setPickable', args: [ true ] }
      ],

      setInactiveAndNotInteractive:[
        { phetioID: 'forcesAndMotionBasics.sim.activeProperty', method: 'setValue', args: [ false ] },
        { phetioID: 'phetio', method: 'setInteractive', args: [ false ] }
      ],

      setActiveAndInteractive:[
        { phetioID: 'forcesAndMotionBasics.sim.activeProperty', method: 'setValue', args: [ true ] },
        { phetioID: 'phetio', method: 'setInteractive', args: [ true ] }
      ],

      applyPredictions:[
        // Show Checkbox panel, volume control, and reset all button
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.controlPanel', method: 'setVisible', args: [ true ] },
        // Go button
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.goPauseButton', method: 'setVisible', args: [ true ] },
        // Show  left force arrow
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.leftArrow', method: 'setVisible', args: [ true ] },
        // Show right force arrow
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.rightArrow', method: 'setVisible', args: [ true ] },
        // Sim is active
        { phetioID: 'forcesAndMotionBasics.sim.activeProperty', method: 'setValue', args: [ true ] },
        // Sim is interactive
        { phetioID: 'phetio', method: 'setInteractive', args: [ true ] },
        // Reset state
        { phetioID: 'forcesAndMotionBasics.netForceScreen.model', method: 'reset', args: [] },
        // Hide the reset all button
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.controlPanel.resetAllButton', method: 'setVisible', args: [ false ] },
        // Hide the return button
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.returnButton', method: 'setVisible', args: [ false ] }
      ],


      applyRestoreCase:[
        // Hide checkbox panel, volume control, and reset all button
        {phetioID: 'forcesAndMotionBasics.netForceScreen.view.controlPanel',method: 'setVisible',args: [ false ]},
        // Hide Go button
        {phetioID: 'forcesAndMotionBasics.netForceScreen.view.goPauseButton',method: 'setVisible',args: [ false ]},
        // Hide Return button
        {phetioID: 'forcesAndMotionBasics.netForceScreen.view.returnButton',method: 'setVisible',args: [ false ]},
        // Hide left force arrow
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.leftArrow', method: 'setVisible', args: [ false ] },
        // Hide right force arrow
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.rightArrow', method: 'setVisible', args: [ false ] },
      ]

    },


    /**
     * This is in a function because these customizations rely on window.classroom.apply.readyToTestCases
     * which doesn't exist when this script is loaded
     *
     * @returns {[]} -  an array of the customization (expressions) to be invoked
     */
    getApplyOnLoadCustomizations: function(){
      return [
        { phetioID: 'forcesAndMotionBasics.sim.activeProperty', method: 'setValue', args: [ window.classroom.apply.startedToTestCases ] },
        { phetioID: 'phetio', method: 'setInteractive', args: [window.classroom.apply.startedToTestCases] },
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.controlPanel', method: 'setVisible', args: [ window.classroom.apply.startedToTestCases  ] },
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.goPauseButton', method: 'setVisible', args: [ window.classroom.apply.startedToTestCases  ] },
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.returnButton', method: 'setVisible', args: [ false ] },
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.controlPanel.resetAllButton', method: 'setVisible', args: [ false  ] },
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.leftArrow', method: 'setVisible', args: [ window.classroom.apply.startedToTestCases ] },
        { phetioID: 'forcesAndMotionBasics.netForceScreen.view.rightArrow', method: 'setVisible', args: [ window.classroom.apply.startedToTestCases ] }
      ];
    },

    enableValuesForCases: function(){
      window.simIFrameClient.invoke( 'forcesAndMotionBasics.netForceScreen.model.showValuesProperty', 'setValue', [ window.classroom.apply.startedToTestCases ] );
    },

      //@formatter:on

    // Individual tandems that did not need sequences to be factored out
    modelTandem: 'forcesAndMotionBasics.netForceScreen.model',
    activePropertyTandem: 'forcesAndMotionBasics.sim.activeProperty',

    /**
     * These are set in experiment, but are listeners for all tabs
     */
    initializeSimInteractionListeners: function() {
      window.simIFrameClient.invoke( 'forcesAndMotionBasics.netForceScreen.model.runningProperty', 'link', [ window.simConfiguration.runningListener ] );
      window.simIFrameClient.invoke( 'forcesAndMotionBasics.netForceScreen.model.startedProperty', 'link', [ window.simConfiguration.startedListener ] );
      window.simIFrameClient.invoke( 'forcesAndMotionBasics.sim.activeProperty', 'link', [ window.simConfiguration.activeListener ] );
      window.simIFrameClient.invoke( 'forcesAndMotionBasics.netForceScreen.model.stateProperty', 'link', [ window.simConfiguration.stateListener ] );
    },

    /**
     * The customizations for experiment ideas, differing based if the idea is tested or not
     * @param ideaNumber
     * @returns {[]} - an array of the customization (expressions) to be invoked
     */
    getIdeaCustomizations: function( ideaNumber ) {
      return window.simConfiguration.customization.enablePullers
        .concat( [
          { phetioID: 'forcesAndMotionBasics.netForceScreen.model', method: 'reset' },
          {
            phetioID: 'forcesAndMotionBasics.sim.activeProperty',
            method: 'setValue',
            args: [ !window.classroom.experiment.ideas[ ideaNumber - 1 ].tested ]
          },
          {
            phetioID: 'forcesAndMotionBasics.netForceScreen.model.showValuesProperty',
            method: 'setValue',
            args: [ true ]
          }
        ] );
    },


    // Table column header text for experiment tab
    experimentTableHeaders: [
      { text: 'Trial', width: 35 },
      { text: 'Restore Trial', width: 135 },
      { text: 'Force Values', width: 120 },
      { text: 'Sum of Forces', width: 120 },
      { text: 'Result', width: 120 },
      { text: 'Time to Win', width: 120 },
      { text: 'Delete', width: 0 }
    ],


    /**
     * The logic in populating each cell of the Experiment data table and the Reflect data table
     * @param dataTable - the data table to apply the setting methods to
     * @returns {Function[]} - an array of functions ordered, one for each column left to right
     */
    getExperimentCellSetters: function( dataTable ) {
      return [
        dataTable.trialNumberCellSetter,

        // Restore Sim cell setter
        function( trialNumber, cell, dataPoint){
          dataTable.restoreSimCellSetter( trialNumber, cell, dataPoint, function(){

            // This is a hack because restoring the sim changes the active property also. So we have to overwrite that
            // If the experiment idea has been tested, or if the reflect tab is not active
            if(window.classroom.experiment.ideas[ window.classroom.experiment.currentIdeaIndex].tested) {
              window.simIFrameClient.invokeSequence( window.simConfiguration.customization.setActiveAndInteractive );
            }
          });
        },

        // Set the L/R force values
        function( trialNumber, cell, dataPoint ) {
          var outerSpan = document.createElement( 'span' );
          cell.classList.add( 'values-arrows' );

          var leftImg = document.createElement( 'img' );
          leftImg.src = window.simConfiguration.getImgSrc( 'img/force-arrows/left-' + Math.abs( dataPoint.leftForce ) + '-grey.png', 440 );
          var rightImg = document.createElement( 'img' );
          rightImg.src = window.simConfiguration.getImgSrc( 'img/force-arrows/right-' + Math.abs( dataPoint.rightForce ) + '-grey.png', 440 );

          outerSpan.appendChild( leftImg );
          outerSpan.appendChild( rightImg );

          cell.appendChild( outerSpan );
        },

        // Set the Net Force value
        function( trialNumber, cell, dataPoint ) {
          var outerSpan = document.createElement( 'span' );
          cell.classList.add( 'values-arrows' );

          if ( Math.abs( dataPoint.netForce ) > 1E-6 ) {
            var direction = dataPoint.netForce > 0 ? 'right' : 'left';
            var netImg = document.createElement( 'img' );
            var srcString = 'img/force-arrows/' + direction + '-' + Math.abs( dataPoint.netForce ) + '-green.png';
            netImg.src = window.simConfiguration.getImgSrc( srcString, 440 );
            outerSpan.appendChild( netImg );
          }
          else {
            var textSpan = document.createElement( 'span' );
            textSpan.innerHTML = '0 N';
            outerSpan.appendChild( textSpan );
          }
          cell.appendChild( outerSpan );
        },

        // Show the winning condition flag
        function resultCellSetter( trialNumber, cell, dataPoint ) {
          if ( dataPoint.modelState === 'completed' ) {
            var e = document.createElement( 'span' );
            var srcString = '';
            if ( dataPoint.netForce === 0 ) {
              srcString = 'img/tie-conditions.png';
              if ( dataPoint.leftForce === 0 && dataPoint.rightForce === 0 ) {
                // This means that the Go! button was never pressed. No experiment was ever conducted.
                return;
              }
            }
            else if ( -1 * dataPoint.leftForce > dataPoint.rightForce ) {
              srcString = 'img/blue-wins.png';
            }
            else if ( -1 * dataPoint.leftForce < dataPoint.rightForce ) {
              srcString = 'img/red-wins.png';
            }
            var img = document.createElement( 'img' );
            img.src = window.simConfiguration.getImgSrc( srcString, 320 );
            img.classList.add( 'winner-flag' );
            e.appendChild( img );
            cell.appendChild( e );
          }
        },

        // Show the time to reach the decision
        function timeToWinCellSetter( trialNumber, cell, dataPoint ) {
          var e = document.createElement( 'span' );
          var p = document.createElement( 'p' );
          if ( dataPoint.modelState === 'experimenting' ) {
            p.innerHTML = '';
          }
          else if ( dataPoint.netForce === 0 ) {
            p.innerHTML = '-';
          }
          else {
            p.innerHTML = dataPoint.duration.toFixed( 1 ) + ' s';
          }
          e.appendChild( p );
          cell.appendChild( e );
        },
        dataTable.deleteButtonCellSetter
      ];
    },

    getImgSrc: function( imgSrc, width ) {
      if ( !window.classroom.apply.printingReport ) {
        return imgSrc;
      }
      var canvas = document.createElement( 'canvas' );
      var origWidth = canvas.width;
      var origHeight = canvas.height;
      canvas.width = width;
      canvas.height = width * origHeight / origWidth;
      var context = canvas.getContext( '2d' );
      var image = new Image(); // eslint-disable-line no-html-constructors
      image.src = imgSrc;
      context.drawImage( image, 0, 0, canvas.width, canvas.height );
      return canvas.toDataURL();
    },


    // Table column header text
    applyTableHeaders: [
      { text: 'Case', width: 40 },
      { text: 'Restore Case', width: 145 },
      { text: 'My Prediction', width: 155 },
      { text: 'My Explanation', width: 225 },
      { text: 'Result', width: 110 }
    ],

    /**
     * The logic in populating each cell of the Apply data table
     * @returns {Function[]} - an array of functions ordered, one for each column left to right
     */
    getApplyCellSetters: function() {
      return [
        // Set case number
        function( caseNumber, cell ) {
          var sp = document.createElement( 'span' );
          sp.innerHTML = caseNumber;
          cell.appendChild( sp );
        },

        // Restore Case
        function( caseNumber, cell ) {
          cell.classList.add( 'screenshot' );

          var image = document.createElement( 'img' );
          image.src = 'img/apply-case' + caseNumber + '.png';
          image.classList.add( 'thumbnail' );
          cell.appendChild( image );

          var button = document.createElement( 'i' );
          button.setAttribute( 'aria-hidden', 'true' );
          button.setAttribute( 'class', 'fa' );
          button.className += ' fa-play-circle';
          button.id = 'apply-table-restore-' + caseNumber;
          button.trialNumber = caseNumber;
          cell.appendChild( button );

          $( cell ).click( function() {
            window.classroom.apply.currentCase = caseNumber;
            $( 'table.apply-table .activeRow' ).removeClass( 'activeRow' );
            $( '#trialTableRow' + caseNumber ).addClass( 'activeRow' );
            window.simIFrameClient.invoke( 'forcesAndMotionBasics.netForceScreen.model', 'reset', [], function() {
              window.simIFrameClient.invokeSequence( window.simConfiguration.customization.applyCases[ caseNumber - 1 ], function() {
                window.simIFrameClient.invokeSequence( window.simConfiguration.customization.disablePullers,
                  function() { window.simConfiguration.enableValuesForCases(); }
                );
              } );
            } );
            window.recordButtonPressed( this );
          } );
        },

        // Prediction
        function( caseNumber, cell ) {
          var sp = document.createElement( 'span' );
          switch( window.classroom.apply.cases[ caseNumber - 1 ].prediction ) {
            case 'red':
              sp.innerHTML = 'Red team wins';
              break;
            case 'blue':
              sp.innerHTML = 'Blue team wins';
              break;
            case 'tie':
              sp.innerHTML = 'The cart doesn\'t move';
              break;
            default:
              console.log( 'Prediction did not have an expected value' );
              return false;
          }
          cell.appendChild( sp );
        },

        // Explanation
        function( caseNumber, cell ) {
          var sp = document.createElement( 'span' );
          sp.innerHTML = window.classroom.apply.cases[ caseNumber - 1 ].explanation;
          cell.appendChild( sp );
        },

        // Winning flag, begins blank
        function( caseNumber, cell ) {
          var flagSpan = document.createElement( 'span' );
          flagSpan.id = 'apply-table-winning-flag' + caseNumber;
          cell.classList.add( 'screenshot' );

          var flagSrc = window.classroom.apply.cases[ caseNumber - 1 ].winningFlagImgSrc;
          if ( flagSrc ) {
            var flagImg = document.createElement( 'img' );
            flagImg.src = flagSrc;
            flagSpan.appendChild( flagImg );
          }
          cell.appendChild( flagSpan );
        }
      ];
    },

    // Holds the last known data point when runningProperty was toggled
    currentDataPoint: {
      leftForce: 0,
      rightForce: 0,
      netForce: 0,
      duration: 0
    },

    // Holds a flag that is toggled when the simulation startedProperty changes
    started: false,
    running: false,
    state: 'experimenting',

    /**
     * If we are on an inactive tab, disable interaction and activity in all sim states
     *
     */
    activeListener: function( active ) {
      if ( !isTestingAllowed() && active ) {
        window.simIFrameClient.invokeSequence( window.simConfiguration.customization.setInactiveAndNotInteractive );
        return false;
      }
      else {
        return true;
      }
    },

    /**
     *  Disables the record button when the Go button is pressed.
     *  @param started {boolean} - boolean passed from forces-and-motion-basics started property
     */
    startedListener: function( started ) {
      if ( isTestingAllowed() ) {
        window.simConfiguration.started = started;
        if ( !window.classroom.experiment.isRecordDisabled ) {
          window.classroom.experiment.isRecordDisabled = true;
          window.simIFrameClient.invoke( 'phetio', 'setInteractive', [ true ] );
        }

        // If not strated or running, re-enable the record buton
        if ( !started && !window.simConfiguration.running ) {
          window.simIFrameClient.invokeSequence( window.simConfiguration.customization.enablePullers );
          if ( window.classroom.currentTabId === 'explore' ) {
            window.simIFrameClient.invoke( window.simConfiguration.activePropertyTandem, 'setValue', [ true ] );
          }
        }
      }
    },


    /**
     * If we are in a completed state, disable interaction with the pullers until return or reset is pressed.
     * We do not want to record an unbalanced state in a win/tie condition
     * @param state {'completed'|'experimenting'}
     */
    stateListener: function( state ) {
      window.simConfiguration.state = state;

      if ( window.classroom.currentTabId === 'explore' ) {
        if ( state === 'experimenting' ) {
          $( '#explore-time-to-win' ).text( '' );
        }
      }
      if ( state === 'completed' ) {
        window.simIFrameClient.invokeSequence( window.simConfiguration.customization.disablePullers );

        // WIN CONDITIONS MET
        var $recordButton = $( '#phet-data-table-record' );
        if ( $recordButton[ 0 ] ) {
          window.simConfiguration.getForceAndDuration( function() {
            // Enable interaction
            window.simIFrameClient.invoke( 'phetio', 'setInteractive', [ true ] );
            // enable the record button and save the current data
            window.classroom.experiment.isRecordDisabled = false;
          } );
        }
        else if ( window.classroom.apply.readyToTestCases && window.classroom.currentTabId === 'apply' ) {
          window.classroom.apply.casesComplete.add( window.classroom.apply.currentCase );
          var span = document.getElementById( 'apply-table-winning-flag' + window.classroom.apply.currentCase );
          if ( !span.hasChildNodes() ) {
            var img = document.createElement( 'img' );
            var caseIndex = window.classroom.apply.currentCase - 1;
            if ( Math.abs( window.simConfiguration.currentDataPoint.netForce ) < 1E-6 ) {
              img.src = 'img/tie-conditions.png';
              window.classroom.apply.cases[ caseIndex ].winningFlagImgSrc = 'img/tie-conditions.png';
            }
            else if ( window.simConfiguration.currentDataPoint.netForce > 0 ) {
              img.src = 'img/red-wins.png';
              window.classroom.apply.cases[ caseIndex ].winningFlagImgSrc = 'img/red-wins.png';
            }
            else {
              img.src = 'img/blue-wins.png';
              window.classroom.apply.cases[ caseIndex ].winningFlagImgSrc = 'img/blue-wins.png';
            }
            span.appendChild( img );
          }
        }
        else if ( window.classroom.currentTabId === 'explore' ) {
          window.simConfiguration.getForceAndDuration( function() {
            var timeToWin = '';
            if ( Math.abs( window.simConfiguration.currentDataPoint.netForce ) < 1E-6 ) {
              timeToWin = '-';
            }
            else {
              timeToWin = window.simConfiguration.currentDataPoint.duration.toFixed( 1 ) + ' seconds';
            }
            $( '#explore-time-to-win' ).text( timeToWin );
          } );
        }
      }

      else {
        window.simIFrameClient.invokeSequence( window.simConfiguration.customization.enablePullers );
      }
    },

    /**
     * When running is toggled, we check to see if we are in a win or tie state, record data, and enable the record button
     * @param running {boolean} - given by the model.runningProperty
     */
    runningListener: function( running ) {
      window.simConfiguration.running = running;
      if ( isTestingAllowed() ) {
        window.simConfiguration.getForceAndDuration( function() {
          if ( window.simConfiguration.started ) {
            if ( running ) {

              // TIE CONDITIONS MET
              // This statement will turn the state to 'completed' so that the stateListener
              // recognizes that a tie is finished 'winning' condition.
              if ( window.simConfiguration.currentDataPoint.netForce === 0 && window.simConfiguration.currentDataPoint.velocity === 0 ) {

                // Emulate win conditions and enable interaction
                window.simIFrameClient.invokeSequence( [
                  {
                    phetioID: 'forcesAndMotionBasics.netForceScreen.model.stateProperty',
                    method: 'setValue',
                    args: [ 'completed' ]
                  },
                  {
                    phetioID: 'forcesAndMotionBasics.netForceScreen.model.runningProperty',
                    method: 'setValue',
                    args: [ false ]
                  },
                  { phetioID: 'phetio', method: 'setInteractive', args: [ true ] }
                ] );
              }

              // Started and running but not in tie, disable the pullers and the recording button.
              else {
                window.simIFrameClient.invokeSequence( window.simConfiguration.customization.disablePullers );
              }
            }
            // Started but not running
            else {
              // If the pause buttton is pressed, the sim will not be running and it will no be completed
              if ( window.classroom.currentTabId === 'explore' && window.simConfiguration.state === 'experimenting' ) {
                window.simIFrameClient.invokeSequence( window.simConfiguration.customization.enablePullers );
              }
            }

          }
          // User hit the return button so enablePullers
          else if ( !running ) {
            window.simIFrameClient.invokeSequence( window.simConfiguration.customization.enablePullers );
            if ( window.classroom.currentTabId === 'explore' ) {
              window.simIFrameClient.invoke( window.simConfiguration.activePropertyTandem, 'setValue', [ true ] );
            }
          }
        } );
      }
    },

    /**
     * Used by the state and running listeners to get data from the sim
     * This data is returned in the 'window.simConfiguration.currentDataPoint' object
     * @callback callback
     */
    getForceAndDuration: function( callback ) {
      window.simIFrameClient.invokeSequence( [
          { phetioID: 'forcesAndMotionBasics.netForceScreen.model.durationProperty', method: 'getValue' },
          { phetioID: 'forcesAndMotionBasics.netForceScreen.model.cart.vProperty', method: 'getValue' },
          { phetioID: 'forcesAndMotionBasics.netForceScreen.model.leftForceProperty', method: 'getValue' },
          { phetioID: 'forcesAndMotionBasics.netForceScreen.model.rightForceProperty', method: 'getValue' },
          { phetioID: 'forcesAndMotionBasics.netForceScreen.model.netForceProperty', method: 'getValue' }
        ], function( duration, velocity, leftForce, rightForce, netForce ) {
          window.simConfiguration.currentDataPoint = {
            leftForce: leftForce,
            rightForce: rightForce,
            netForce: netForce,
            duration: duration,
            velocity: velocity
          };
          callback();
        }
      );

    },

    /**
     *  Record a data point in the data table, only if the record button is not disabled
     * @param simIFrameClient - a deprecated relic from other wrappers, we use window.simIFrameClient
     * @param {function} callback
     * @callback callback - returns the recorded dataPoint through the callback
     */
    recordDataPoint: function( simIFrameClient, callback ) {

      // Get the screenshot and state
      var phetioSequence = [
        { phetioID: 'forcesAndMotionBasics.netForceScreen.model.leftForceProperty', method: 'getValue' },
        { phetioID: 'forcesAndMotionBasics.netForceScreen.model.rightForceProperty', method: 'getValue' },
        { phetioID: 'forcesAndMotionBasics.netForceScreen.model.netForceProperty', method: 'getValue' },
        { phetioID: 'forcesAndMotionBasics.netForceScreen.model.stateProperty', method: 'getValue' },
        { phetioID: 'forcesAndMotionBasics', method: 'getScreenshotDataURL' },
        { phetioID: 'phetio', method: 'getState' }
      ];

      window.simIFrameClient.invokeSequence( phetioSequence,
        function( leftForce, rightForce, netForce, modelState, screenshot, state ) {
          var image = document.createElement( 'img' );
          image.className = 'thumbnail';
          image.src = screenshot;

          // Record the data on the dataTable
          var dataPoint = {
            leftForce: leftForce,
            rightForce: rightForce,
            netForce: netForce,
            duration: window.simConfiguration.currentDataPoint.duration,
            image: image,
            state: state,
            modelState: modelState
          };
          callback( dataPoint );
        }
      );
    }


  };
})();