// Copyright 2016, University of Colorado Boulder

(function() {
  'use strict';

  /**
   * Enables the save button if true
   * @returns {boolean} - true if textarea is not empty
   */
  function validateExplore() {
    // Check that there is a radio button selected and that the textarea is not blank
    var flag = !!$( '#explore-input' ).val();

    // Enable/Disable the submit button if needed
    var submitButton = document.getElementById( 'explore-submit' );
    if ( flag && submitButton.classList.contains( 'disabled' ) ) {
      submitButton.classList.remove( 'disabled' );
    }
    else if ( !flag && !submitButton.classList.contains( 'disabled' ) ) {
      submitButton.classList.add( 'disabled' );
    }

    return flag;
  }

  /**
   * If text exists, toggles the save/edit buttons and enables the next tab
   */
  function onExploreSubmit() {
    if ( validateExplore() ) {
      window.classroom.predict.notes = this.value;
      $( '#explore-submit' ).hide();
      $( '#explore-edit' ).show();
      $( '#explore-next' ).show();
      $( '#explore-input' ).attr( 'disabled', 'disabled' );
      $( '#experiment' ).attr( 'class', 'tab pickable' );

      window.recordButtonPressed( this );
    }
    else {
      window.toast( 'Please take notes of three observations to proceed.' );
    }
  }


  /**
   * If "Edit" is clicked, re-enable the text area and toggle the save/edit buttons
   * @returns {boolean} - false always.  This allows button to be pickable in the form but does not submit a post request
   */
  function onExploreEdit() {
    $( '#explore-submit' ).show();
    $( '#explore-edit' ).hide();
    validateExplore();
    $( '#explore-input' ).removeAttr( 'disabled' );

    window.recordButtonPressed( this );

    return false;
  }

  /**
   * Life cycle functions for this tab.
   * @type {{initialize: tabs.experiment.initialize, onLoad: tabs.experiment.onLoad, onLeave: tabs.experiment.onLeave, afterStateLoad: tabs.experiment.afterStateLoad}}
   */
  window.tabs.explore = {
    /**
     * Initial set up of the tab. Sets the click listeners and prepares the sim accordingly
     * Called from classroom-activity.switchTab.
     */
    initialize: function() {

      /**
       * Initialize listeners.
       *
       * These do not work in simIFrameClient.invokeSequence for reasons yet unknown,
       * and do not work if wired up in classroom-activity.js
       *
       * These are initialized in explore because it is the first time that we
       * need to pull info out of the sim. They carry on throughout the sim.
       **/
      window.simConfiguration.initializeSimInteractionListeners();


      $( '#explore-submit' ).click( onExploreSubmit );

      var $edit = $( '#explore-edit' );
      $edit.click( onExploreEdit );
      $edit.hide();  // Edit button should start hidden

      // Listener for textarea changes on the explore tab
      $( '#explore-input' ).on( 'keyup change paste', function() {
        validateExplore();

        window.classroom.explore.notes = window.recordTextAreaChanged( this, window.classroom.explore.notes );

        return true;
      } );

      $( '#explore-text' ).text( window.simConfiguration.exploreText );

      this.onLoad();
    },

    /**
     * Sets the properties of the classroom object and customizations of the sim
     * that must be done each time traveling to this tab.
     * Called from classroom-activity.switchTab whenever going to this tab
     */
    onLoad: function() {
      $( '#explore-time-to-win-div').show();

      if (window.classroom.activeTabId !== 'explore'){
        $( '#explore-submit' ).click();
        $( '#explore-edit' ).hide();
      }
    },

    /**
     * Called from classroom-activity.switchTab whenever leaving this tab and before loading/initializing the next
     */
    onLeave: function() {
      window.simIFrameClient.invoke( window.simConfiguration.activePropertyTandem, 'setValue', [ false ] );
      $( '#explore-time-to-win-div').hide();
    },

    /**
     * Called from classroom-activity.switchTab after onLoad and this tab's customizations have been invoked and loaded
     */
    afterStateLoad: function() {}
  };
})();