// Copyright 2016, University of Colorado Boulder

// Loaded by lab-book.js
(function() {
  'use strict';

  window.simConfiguration = {

    pageTitle: 'Lab Book - Capacitor Lab: Basics',
    simTitle: 'Capacitor Lab',
    simName: 'capacitorLabBasics',

    queryString: 'screens=2&switch=twoState',

    customization: [

      // Hide voltmeter and its toolbox
      {
        phetioID: 'capacitorLabBasics.lightBulbScreen.view.voltmeterToolBox',
        method: 'setVisible',
        args: [ false ]
      },

      // Hide capacitance meter checkbox
      {
        phetioID: 'capacitorLabBasics.lightBulbScreen.view.barMeterPanel.capacitanceMeterCheckBox',
        method: 'setVisible',
        args: [ false ]
      },

      // Hide capacitance bar meter
      {
        phetioID: 'capacitorLabBasics.lightBulbScreen.model.capacitanceMeterVisibleProperty',
        method: 'setValue',
        args: [ false ]
      },

      // Hide stored energy bar meter
      {
        phetioID: 'capacitorLabBasics.lightBulbScreen.view.barMeterPanel.storedEnergyMeterCheckBox',
        method: 'setVisible',
        args: [ false ]
      },

      // Hide checkbox to show bar meters
      {
        phetioID: 'capacitorLabBasics.lightBulbScreen.view.viewControlPanel.verticalCheckBoxGroup.barGraphsCheckBox',
        method: 'setVisible',
        args: [ false ]
      },

      // Check plate charge checkbox
      {
        phetioID: 'capacitorLabBasics.lightBulbScreen.view.barMeterPanel.plateChargeMeterCheckBox',
        method: 'setChecked',
        args: [ true ]
      },

      // Disable pickability on plate charge checkbox
      {
        phetioID: 'capacitorLabBasics.lightBulbScreen.view.barMeterPanel.plateChargeMeterCheckBox',
        method: 'setPickable',
        args: [ false ]
      },

      // Change 1.5V label to '+'
      {
        phetioID: 'capacitorLabBasics.lightBulbScreen.view.lightBulbCircuitNode.batteryNode.maximumTickLabel',
        method: 'setText',
        args: [ '+' ]
      },

      // Change -1.5V label to '-'
      {
        phetioID: 'capacitorLabBasics.lightBulbScreen.view.lightBulbCircuitNode.batteryNode.minimumTickLabel',
        method: 'setText',
        args: [ '-' ]
      },

      // Increase font size of '+' label
      {
        phetioID: 'capacitorLabBasics.lightBulbScreen.view.lightBulbCircuitNode.batteryNode.maximumTickLabel',
        method: 'setFontOptions',
        args: [ { size: 32 } ]
      },

      // Increase font size of '0 V' label
      {
        phetioID: 'capacitorLabBasics.lightBulbScreen.view.lightBulbCircuitNode.batteryNode.defaultTickLabel',
        method: 'setFontOptions',
        args: [ { size: 24, weight: 'bold' } ]
      },

      // Increase font size of '-' label
      {
        phetioID: 'capacitorLabBasics.lightBulbScreen.view.lightBulbCircuitNode.batteryNode.minimumTickLabel',
        method: 'setFontOptions',
        args: [ { size: 32 } ]
      },

      // Increase width of plate charge meter text box so the font size can be increased
      {
        phetioID: 'capacitorLabBasics.lightBulbScreen.view.barMeterPanel.plateChargeMeterNode.valueText',
        method: 'setMaxWidth',
        args: [ 75 ] // Default is 45px
      },

      // Increase font size of plate charge meter text
      {
        phetioID: 'capacitorLabBasics.lightBulbScreen.view.barMeterPanel.plateChargeMeterNode.valueText',
        method: 'setFontOptions',
        args: [ { size: 18 } ]
      },

      // Hide reset button
      {
        phetioID: 'capacitorLabBasics.lightBulbScreen.view.resetAllButton',
        method: 'setVisible',
        args: [ false ]
      },

      // Hide upper-right view control panel altogether
      {
        phetioID: 'capacitorLabBasics.lightBulbScreen.view.viewControlPanel',
        method: 'setVisible',
        args: [ false ]
      }
    ],

    // To add a listener to reset the customizations on reset.
    resetAllButtonFiringPropertyTandem: 'capacitorLabBasics.lightBulbScreen.view.resetAllButton.isFiringProperty',

    // Table column header text
    tableHeaders: [ {
      text: 'Voltage'
    }, {
      text: 'Separation (mm)'
    }, {
      text: 'Area (mm<sup>2</sup>)'
    }, {
      text: 'Charge (pC)'
    } ],

    // Quantities to plot in chart
    features: [ {
      value: 'trialNumber',
      content: 'Trial Number'
    }, {
      value: 'separation',
      content: 'Separation (mm)'
    }, {
      value: 'area',
      content: 'Area (mm\u00b2)'
    }, {
      value: 'charge',
      content: 'Charge (pC)'
    } ],

    addCells: function( row, dataPoint ) {

      // Add voltage meter bar
      var barDiv = document.createElement( 'div' );
      var id = 'voltage-cell-' + dataPoint.trialNumber;
      barDiv.setAttribute( 'id', id );
      barDiv.setAttribute( 'class', 'voltage-cell' );
      row.insertCell( 1 ).appendChild( barDiv );

      // Note: I found it necessary to pass in the dimensions because when
      // the table is rolled up, the dimensions computed from the
      // parent div using getBoundingClientRect() are returned as 0 x 0.
      window.voltageBar( '#' + id, dataPoint.voltage, {
        width: 40,
        height: 60
      } );

      // Add text for plate separation, area, charge
      row.insertCell( 2 ).innerHTML = Number( dataPoint.separation ).toFixed( 2 );
      row.insertCell( 3 ).innerHTML = Number( dataPoint.area ).toFixed( 2 );
      row.insertCell( 4 ).innerHTML = Number( dataPoint.charge ).toFixed( 2 ); // Charge in pC
    },

    recordDataPoint: function( simIFrameClient, callback ) {
      var sequence = [ {
        phetioID: 'capacitorLabBasics.lightBulbScreen.model.circuit.capacitor.plateVoltageProperty',
        method: 'getValue'
      }, {
        phetioID: 'capacitorLabBasics.lightBulbScreen.model.circuit.capacitor.plateSeparationProperty',
        method: 'getValue'
      }, {
        phetioID: 'capacitorLabBasics.lightBulbScreen.model.circuit.capacitor.plateSizeProperty',
        method: 'getValue'
      }, {
        phetioID: 'capacitorLabBasics.lightBulbScreen.model.plateChargeMeter.valueProperty',
        method: 'getValue'
      }, {
        phetioID: 'capacitorLabBasics.sim',
        method: 'getScreenshotDataURL'
      }, {
        phetioID: 'phetio',
        method: 'getState'
      } ];

      simIFrameClient.invokeSequence( sequence,
        function( voltage, separation, plateSize, charge, screenshot, state ) {
          var image = document.createElement( 'img' );
          image.className = 'thumbnail';
          image.src = screenshot;

          // plateSize stores dimensions in meters. Compute plate area in mm^2
          var area = 1e6 * ( plateSize.maxX - plateSize.minX ) * ( plateSize.maxZ - plateSize.minZ );

          var dataPoint = {
            voltage: voltage / 1.5, // Divide by max for qualitative representation
            separation: 1000 * separation, // m -> mm
            area: area,
            charge: 1e12 * charge, // C -> pC
            image: image,
            state: state,
            visible: false
          };

          callback( dataPoint );
        } );
    }
  };
})();
